require 'serialport'

class Duino
  def initialize(port, baudrate=115200)
    data_bits = 8
    stop_bits = 1
    parity = SerialPort::NONE

    @serial = SerialPort.new(port, baudrate, data_bits, stop_bits, parity)
    @serial.read_timeout = 2
    @serial.sync
    @port         = port
    @output_pins  = []
    @input_pins   = []
    @pin_states   = {}

    puts "Initialized"
  end
  
  def to_s
    "Arduino is on port #{@port} at #{@serial.baud} baudrate"
  end
    
  def digital_output(*pin_list)
  send_data(pin_list.length)
    if(pin_list.class == Array)
      @output_pins = pin_list
      pin_list.each do |pin|
        send_pin(pin)
      end
    else
      raise ArgumentError, "Arguments must be a list of pin numbers"
    end
    return pin_list
  end
    
  def digital_input(*pin_list)
    send_data(pin_list.length)
    if(pin_list.class == Array)
      @input_pins = pin_list
      pin_list.each do |pin|
        send_pin(pin)
      end
    else
      raise ArgumentError, "Arguments must be a list of pin numbers"
    end
    return pin_list
  end
    
  def set_high(pin)
    save_state(pin, true)
    send_data(1)
    send_pin(pin)
  end
    
  def set_low(pin)
    save_state(pin, false)
    send_data(0)
    send_pin(pin)
  end
    
  def save_state(pin, state)
    @pin_states[pin.to_s] = state
  end
    
  def get_state(pin) # Returns true if high and false if low.
    send_data(2)
    send_pin(pin)

    while true
      rval = @serial.getbyte()
      if(rval == 0)
       return("Low")
      end
      if(rval == 1)
       return("High")
      end
    end
  end
    
  def analog_write(pin, value)
    send_data(3)
    send_pin(pin)
    send_data((value / 16).to_s(16))
    send_data((value % 16).to_s(16))
  end
    
  def analog_read(pin)
    send_data(4)
    send_pin(pin)
    get_data()
  end
 
  def turn_off
    @output_pins.each do |pin|
      set_low(pin)
    end
    puts "All pins set to low"
  end
  
  def close
    send_data(5)
    @serial.dtr = (0)
    @serial.close
    puts "Connection closed"
  end
  
  private
  
  def send_pin(pin)
    send_data(pin)
  end

  def send_data(serial_data)
    while true
      break if(get_data() == "?")
    end
    @serial.write serial_data.chr
  end
  
  def get_data
    clean_data = @serial.readlines()
    clean_data = clean_data.join("").gsub("\n","").gsub("\r","")
  end
end
